#include "I2CBus.h"
#include <stdio.h>

extern uint32_t _load_start_I2CDriver_cog[];
extern uint32_t _load_stop_I2CDriver_cog[];
extern uint8_t binary_i2c_driver_image_dat_start[];
int32_t  pollDelayClocks;

#define pollDelay()    (waitcnt(pollDelayClocks+CNT))

I2C::I2C(int scl, int sda, int freq)
{  
    I2C_INIT init;

    m_dev.cog            = -1;
    m_attachCnt          = 0;
    init.scl             = scl;
    init.sda             = sda;
    init.ticks_per_cycle = (CLKFREQ / freq)-38;
    init.mailbox         = &m_dev.mailbox;
    m_dev.mailbox.cmd    = I2C_CMD_INIT;
    pollDelayClocks      = CLKFREQ/80000;   // About 10 uSecs
    
    // only allow speeds up to 400khz for now
    if (freq > 400000)
        return;
    
    // Start the cog according to the method needed for LMM/XMM memory models
    #if defined(__PROPELLER_XMMC__) || defined(__PROPELLER_XMM__)
        int size = _load_stop_I2CDriver_cog - _load_start_I2CDriver_cog;
        unsigned int cogbuffer[size];
        // memcpy does bytes, so we copy numbytes * 4 = num ints.      
        memcpy(cogbuffer, _load_start_I2CDriver_cog, size<<2);
    #else
        int *cogbuffer = (int*)_load_start_I2CDriver_cog;
    #endif
    
    m_dev.cog = cognew(cogbuffer, &init);   

    volatile int n;
    while (m_dev.mailbox.cmd != I2C_CMD_IDLE)
    {
        if (n > 254)
        {
            m_ready = 0;
            return;          // Timeout waiting after ~2.5ms           
        }
        pollDelay();        // Wait 10 uSecs.
        n++;
    }

    m_ready = 1;
}


I2C::~I2C()
{
    if ((m_dev.cog >= 0) && (m_attachCnt<1))
            cogstop(m_dev.cog);
    
    m_dev.cog = -1;    
}


int I2C::GetCog()
{// Returns the current COG that is driving the bus.  0-7 are valid
 // values. All others indicate the COG is not started. 
 
    return m_dev.cog;
}


int I2C::DevPresent(uint8_t addr)
{// Write out the device write address to the bus with no register and
 // no data bytes.  If there is an ACK we know the device is present
 // on the bus.  If no ACK then it may not be present or may be busy.
    if(m_ready)
    {
        while (m_dev.mailbox.cmd != I2C_CMD_IDLE)
            pollDelay();

        m_dev.mailbox.hdr   = (addr << 1);
        m_dev.mailbox.count = 0;
        m_dev.mailbox.reg_count = 0;
        m_dev.mailbox.cmd   = I2C_CMD_SEND;

        while (m_dev.mailbox.cmd != I2C_CMD_IDLE)
            pollDelay();

        return m_dev.mailbox.sts == I2C_OK ? 1 : 0;
    }
    else
        return 0;
}


int I2C::GetStatus()
{// Return the status value from the last I2C bus transaction.
    return m_dev.mailbox.sts;
}

int I2C::StartBus()
{
	return -1;
}

volatile I2C_MAILBOX* I2C::Attach()
{// Allows an external program to attach to the COG running
 // this I2C bus directly by providing a pointer to the COG
 // mailbox. 
 // *** Returns NULL if there is no bus COG running! ***
 
    if (m_dev.cog > 0)
    {
        m_attachCnt++;
        return &m_dev.mailbox;
    }
    
    return 0;
}

int I2C::Detach(I2C_MAILBOX* mbox)
{// Detaches the external program 'returning' the mailbox
 // pointer.  Only requests that include this object's
 // mailbox will detach.  Returns 0 for success and -1 for
 // error.  
    if (mbox == &m_dev.mailbox)
    {
        m_attachCnt--;
        return 0;
    }
    
    return -1;
}      


int I2C::TXByte(uint8_t addr, uint8_t &bt)
{// Transmite a single byte of data to the bus.  No register address.
        return TXByte(addr, 0, 0, bt);
}


int I2C::TXByte(uint8_t addr, uint16_t reg, uint8_t rcnt, uint8_t bt)
{// Transmite a single byte of data to the bus at the register value
 // specified.  Transaction is |ST|WRADR|REGVAL|BYTE|SP|.  
    if(m_ready)
    {
        while (m_dev.mailbox.cmd != I2C_CMD_IDLE)
            pollDelay();

        m_dev.mailbox.cmd        = I2C_CMD_LOCKED;
        m_dev.mailbox.hdr        = (addr << 1);
        m_dev.mailbox.buffer     = &bt;
        m_dev.mailbox.count      = 1;
        m_dev.mailbox.reg        = reg;
        m_dev.mailbox.reg_count  = rcnt;
        m_dev.mailbox.cmd        = I2C_CMD_SEND;

        while (m_dev.mailbox.cmd != I2C_CMD_IDLE)
            pollDelay();

        return m_dev.mailbox.sts == I2C_OK ? 0 : -1;
    }
    else
        return -1;
}


 int I2C::TXWord(uint8_t addr, uint16_t &wd)
{
        return TXWord(addr, 0, 0, wd);
}


int I2C::TXWord(uint8_t addr, uint16_t reg, uint8_t rcnt, uint16_t wd)
{// Transmite a single word of data to the bus at the register value
 // specified.  Transaction is |ST|WRADR|REGVAL|BYTE1|BYTE0|SP|.
    uint8_t  bytes[2];
    
    bytes[1] = wd;
    wd >>= 8;
    bytes[0] = wd;  

    if(m_ready)
    {
        while (m_dev.mailbox.cmd != I2C_CMD_IDLE)
            pollDelay();

        m_dev.mailbox.cmd        = I2C_CMD_LOCKED;
        m_dev.mailbox.hdr        = (addr << 1);
        m_dev.mailbox.buffer     = bytes;
        m_dev.mailbox.count      = 2;
        m_dev.mailbox.reg        = reg;
        m_dev.mailbox.reg_count  = rcnt;
        m_dev.mailbox.cmd        = I2C_CMD_SEND;

        while (m_dev.mailbox.cmd != I2C_CMD_IDLE)
            pollDelay();

        return m_dev.mailbox.sts == I2C_OK ? 0 : -1;
    }
    else
        return -1;
}


 int I2C::TXLong(uint8_t addr, uint32_t &lg)
{
    return TXLong(addr, 0, 0, lg);
}


int I2C::TXLong(uint8_t addr, uint16_t reg, uint8_t rcnt, uint32_t lg)
{// Transmite a single long of data to the bus at the register value
 // specified.  Transaction is |ST|WRADR|REGVAL|BYTE3|BYTE2|BYTE1|BYTE0|SP|.
    uint8_t  bytes[4];
    
    for(int i=3; i>=0; i--)
    {
        bytes[i] = lg;
        lg >>= 8;
    } 
    if(m_ready)
    {
        while (m_dev.mailbox.cmd != I2C_CMD_IDLE)
            pollDelay();

        m_dev.mailbox.cmd        = I2C_CMD_LOCKED;
        m_dev.mailbox.hdr        = (addr << 1);
        m_dev.mailbox.buffer     = bytes;
        m_dev.mailbox.count      = 4;
        m_dev.mailbox.reg        = reg;
        m_dev.mailbox.reg_count  = rcnt;
        m_dev.mailbox.cmd        = I2C_CMD_SEND;

        while (m_dev.mailbox.cmd != I2C_CMD_IDLE)
            pollDelay();

        return m_dev.mailbox.sts == I2C_OK ? 0 : -1;
    }
    else
        return -1;
}


int I2C::TXBuf(uint8_t addr, uint8_t *buf, uint8_t count)
{
    return TXBuf(addr, buf, count, 0, 0);
}

int I2C::TXBuf(uint8_t addr, uint8_t *buf, uint8_t count, uint16_t reg, uint8_t rcnt=1)
{
    if(m_ready)
    {
        while (m_dev.mailbox.cmd != I2C_CMD_IDLE)
            pollDelay();

        m_dev.mailbox.cmd       = I2C_CMD_LOCKED;
        m_dev.mailbox.hdr       = (addr << 1);
        m_dev.mailbox.buffer    = buf;
        m_dev.mailbox.count     = count;
        m_dev.mailbox.reg       = reg;
        m_dev.mailbox.reg_count = rcnt;
        m_dev.mailbox.cmd       = I2C_CMD_SEND;

        while (m_dev.mailbox.cmd != I2C_CMD_IDLE)
            pollDelay();

        return m_dev.mailbox.sts == I2C_OK ? 0 : -1;
    }
    else
        return -1;
}






int I2C::RXByte(uint8_t addr)
{// Read a single byte of data from the bus.  No register address.
  
    return RXByte(addr, 0, 0);
}


int I2C::RXByte(uint8_t addr, uint16_t reg, uint8_t rcnt=1)
{// Read a single byte of data from the bus at the register specified.  
 // Register can be 1 or 2 bytes [rcnt]. 
 // Transaction is |ST|WRADR|REGVAL|RS|RDADR|BYTE|SP|.   

    uint8_t bt;

    if(m_ready)
    {
        while (m_dev.mailbox.cmd != I2C_CMD_IDLE)
            pollDelay();

        m_dev.mailbox.cmd       = I2C_CMD_LOCKED;
        m_dev.mailbox.hdr       = (addr << 1);
        m_dev.mailbox.buffer    = &bt;
        m_dev.mailbox.count     = 1;
        m_dev.mailbox.reg       = reg;
        m_dev.mailbox.reg_count = rcnt;
        m_dev.mailbox.cmd       = I2C_CMD_RECEIVE;

        while (m_dev.mailbox.cmd != I2C_CMD_IDLE)
            pollDelay();

        if (m_dev.mailbox.sts == I2C_OK)
            return bt;
        else
            return -1;
    }
    else
        return -1;
}


int I2C::RXWord(uint8_t addr)
{
    return RXWord(addr, 0, 0);
}


int I2C::RXWord(uint8_t addr, uint16_t reg, uint8_t rcnt=1)
{// Read a single word of data from the bus at the register specified.  
 // Register can be 1 or 2 bytes -> [rcnt].  Default is 1 byte. 
 // Transaction is |ST|WRADR|REGVAL|RS|RDADR|BYTE1|BYTE0|SP|.
 
    uint8_t  buf[2];
    uint16_t wd = 0;

    if(m_ready)
    {
        while (m_dev.mailbox.cmd != I2C_CMD_IDLE)
            pollDelay();
        m_dev.mailbox.cmd        = I2C_CMD_LOCKED;
        m_dev.mailbox.hdr        = (addr << 1);
        m_dev.mailbox.buffer     = buf;
        m_dev.mailbox.count      = 2;
        m_dev.mailbox.reg        = reg;
        m_dev.mailbox.reg_count  = rcnt;
        m_dev.mailbox.cmd        = I2C_CMD_RECEIVE;
        while (m_dev.mailbox.cmd != I2C_CMD_IDLE)
            pollDelay();

        if (m_dev.mailbox.sts == I2C_OK)
        {
            wd = buf[0];
            wd <<= 8;
            wd |= buf[1];
        }

        if (m_dev.mailbox.sts == I2C_OK)
            return wd;
        else
            return -1;
    }
    else
        return -1;
}


int I2C::RXLong(uint8_t addr)
{
    return RXLong(addr, 0, 0);
}


int I2C::RXLong(uint8_t addr, uint16_t reg, uint8_t rcnt=1)
{// Read a single long of data from the bus at the register specified.  
 // Register can be 1 or 2 bytes -> [rcnt].  Default is 1 byte. 
 // Transaction is |ST|WRADR|REGVAL|RS|RDADR|BYTE3|BYTE2|BYTE1|BYTE0|SP|.
  
    uint8_t  buf[4];
    uint32_t lg = 0;

    if(m_ready)
    {
        while (m_dev.mailbox.cmd != I2C_CMD_IDLE)
            pollDelay();

        m_dev.mailbox.cmd        = I2C_CMD_LOCKED;
        m_dev.mailbox.hdr        = (addr << 1);
        m_dev.mailbox.buffer     = buf;
        m_dev.mailbox.count      = 4;
        m_dev.mailbox.reg        = reg;
        m_dev.mailbox.reg_count  = rcnt;
        m_dev.mailbox.cmd        = I2C_CMD_RECEIVE;

        while (m_dev.mailbox.cmd != I2C_CMD_IDLE)
            pollDelay();

        if (m_dev.mailbox.sts == I2C_OK)
        {
            lg |= buf[0];
            lg <<= 8;
            lg |= buf[1];
            lg <<= 8;
            lg |= buf[2];
            lg <<= 8;
            lg |= buf[3];
        }

        if (m_dev.mailbox.sts == I2C_OK)
            return lg;
        else
            return -1;
    }
    return -1;
}


int I2C::RXBuf(uint8_t addr, uint8_t *buf, uint8_t count)
{
    return RXBuf(addr, buf, count, 0, 0);
}


int I2C::RXBuf(uint8_t addr, uint8_t *buf, uint8_t count, uint16_t reg, uint8_t rcnt=1)
{
    if(m_ready)
    {
        while (m_dev.mailbox.cmd != I2C_CMD_IDLE)
            pollDelay();

        m_dev.mailbox.cmd       = I2C_CMD_LOCKED;
        m_dev.mailbox.hdr       = (addr << 1);
        m_dev.mailbox.buffer    = buf;
        m_dev.mailbox.count     = count;
        m_dev.mailbox.reg       = reg;
        m_dev.mailbox.reg_count = rcnt;
        m_dev.mailbox.cmd       = I2C_CMD_RECEIVE;

        while (m_dev.mailbox.cmd != I2C_CMD_IDLE)
            pollDelay();

        return m_dev.mailbox.sts == I2C_OK ? 0 : -1;
    }
    else
        return -1;
}
